/* Rev:$Revision: 1.2 $ */
/* Copyright $Date: 2008/05/06 19:36:36 $ */ 
/*******************************************************************************
 *
 *   Copyright 2007 NetBurner, Inc.  ALL RIGHTS RESERVED
 *
 *   Permission is hereby granted to purchasers of NetBurner hardware to use or
 *   modify this computer program for any use as long as the resultant program
 *   is only executed on NetBurner provided hardware.
 *
 *   No other rights to use this program or its derivatives in part or in whole
 *   are granted.
 *
 *   It may be possible to license this or other NetBurner software for use on
 *   non-NetBurner hardware. Please contact licensing@netburner.com for more
 *   information.
 *
 *   NetBurner makes no representation or warranties with respect to the
 *   performance of this computer program, and specifically disclaims any
 *   responsibility for any damages, special or consequential, connected with
 *   the use of this program.
 *
 *   NetBurner, Inc.
 *   5405 Morehouse Drive
 *   San Diego, CA  92121
 *   http://www.netburner.com
 *
 ******************************************************************************/

/*-------------------------------------------------------------------
  Time functions that can be used for EFFS file time stamps.
 ------------------------------------------------------------------*/
#include "predef.h"
#include <stdio.h>
#include <stdlib.h>
#include <startnet.h>
#include <tcp.h>
#include <basictypes.h>
#include <nbtime.h>
#include <time.h>
#ifndef PK70
#include "rtc.h"
#endif
#include <ethernet.h>
#include "config.h"

// Reference to the externally declared AppConfig structure
extern APP_CONFIG AppConfig;

/*-------------------------------------------------------------------
  SetTimeNTP() - Acquires time from a NTP server and sets the system 
  time. For this function to operate correctly you need to have:
  - Access to a time server on your LAN or the Internet
  - Valid IP address, mask and gateway set on your NetBurner device
 -------------------------------------------------------------------*/
BOOL SetTimeNTP()
{
   int i = 0;
   BOOL found = FALSE;
   
   do {
      iprintf("Acquiring time from NTP server: ");
      ShowIP(AppConfig.ntpservers[i]);
      iprintf(" - ");
      if ((found = SetNTPTime(AppConfig.ntpservers[i]))) {
         iprintf( "Complete\r\n" );
      }	else	{
         iprintf( "Failed\r\n" );
         i++;
      }
   } while ((AppConfig.ntpservers[i]) && (!found));
   
   if (!found ) {
   		iprintf("Verify you have correct IP address, mask AND gateway set\r\n");
   }

   return found;
}

/*-------------------------------------------------------------------
  myasctime() - Returns ptr to string containing formatted time. 
-------------------------------------------------------------------*/

char *myasctime(const struct tm *timeptr)
{
    static char wday_name[7][4] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
    static char mon_name[12][4] = {
        "Jan", "Feb", "Mar", "Apr", "May", "Jun",
        "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
    };
    static char result[26];

    sprintf(result, "%.3s %.3s%3d %.2d:%.2d:%.2d %d",
        wday_name[timeptr->tm_wday],
        mon_name[timeptr->tm_mon],
        timeptr->tm_mday, timeptr->tm_hour,
        timeptr->tm_min, timeptr->tm_sec,
        1900 + timeptr->tm_year);
    return result;
}

/*-------------------------------------------------------------------
  DisplaySystemTime
 -------------------------------------------------------------------*/
 void DisplaySystemTime()
 {
    time_t RawTime;
    struct tm *TimeInfo;
 
    time( &RawTime );  // Get system time in time_t format
    TimeInfo = localtime( &RawTime );  // Convert to struct tm format
    iprintf("Time: %s\r\n", myasctime( TimeInfo ) );
}

/*-------------------------------------------------------------------
  SetTimeZone
 -------------------------------------------------------------------*/
 void SetTimeZone( int hour_offset, int isdst )
 {
    time_t RawTime;
    struct tm *TimeInfo;
 
    time( &RawTime );  // Get system time in time_t format
    TimeInfo = localtime( &RawTime );  // Convert to struct tm format
    TimeInfo->tm_hour += hour_offset;
    TimeInfo->tm_isdst = isdst;
    set_time( mktime( TimeInfo ) );    
}



