/* Rev:$Revision: 1.3 $ */
/* Copyright $Date: 2007/09/12 07:37:21 $ */ 
/*******************************************************************************
 *
 *   Copyright 2007 NetBurner, Inc.  ALL RIGHTS RESERVED
 *
 *   Permission is hereby granted to purchasers of NetBurner hardware to use or
 *   modify this computer program for any use as long as the resultant program
 *   is only executed on NetBurner provided hardware.
 *
 *   No other rights to use this program or its derivatives in part or in whole
 *   are granted.
 *
 *   It may be possible to license this or other NetBurner software for use on
 *   non-NetBurner hardware. Please contact licensing@netburner.com for more
 *   information.
 *
 *   NetBurner makes no representation or warranties with respect to the
 *   performance of this computer program, and specifically disclaims any
 *   responsibility for any damages, special or consequential, connected with
 *   the use of this program.
 *
 *   NetBurner, Inc.
 *   5405 Morehouse Drive
 *   San Diego, CA  92121
 *   http://www.netburner.com
 *
 ******************************************************************************/
 
#include <stdio.h>
#include <basictypes.h>
#include <effs_fat/fat.h>
#include <effs_fat/mmc_mcf.h>
#include "filesystemutils.h"

extern "C"
{
   //int get_cd( void );          // Card detection check for MMC cards
   //int get_wp( void );          // Write protection check for MMC cards
}

char driveType[20];
  
char EffsErrorCode[][80] =
{
  "F_NO_ERROR",                // 0
  "F_ERR_INVALIDDRIVE",        // 1   
  "F_ERR_NOTFORMATTED",        // 2
  "F_ERR_INVALIDDIR",          // 3
  "F_ERR_INVALIDNAME",         // 4
  "F_ERR_NOTFOUND",            // 5
  "F_ERR_DUPLICATED",          // 6
  "F_ERR_NOMOREENTRY",         // 7
  "F_ERR_NOTOPEN",             // 8
  "F_ERR_EOF",                 // 9
  "F_ERR_RESERVED",            // 10
  "F_ERR_NOTUSEABLE",          // 11
  "F_ERR_LOCKED",              // 12
  "F_ERR_ACCESSDENIED",        // 13
  "F_ERR_NOTEMPTY",            // 14
  "F_ERR_INITFUNC",            // 15
  "F_ERR_CARDREMOVED",         // 16
  "F_ERR_ONDRIVE",             // 17
  "F_ERR_INVALIDSECTOR",       // 18
  "F_ERR_READ",                // 19
  "F_ERR_WRITE",               // 20
  "F_ERR_INVALIDMEDIA",        // 21
  "F_ERR_BUSY",                // 22
  "F_ERR_WRITEPROTECT",        // 23
  "F_ERR_INVFATTYPE",          // 24
  "F_ERR_MEDIATOOSMALL",       // 25
  "F_ERR_MEDIATOOLARGE",       // 26
  "F_ERR_NOTSUPPSECTORSIZE",   // 27
  "F_ERR_DELFUNC",             // 28
  "F_ERR_MOUNTED",             // 29
  "F_ERR_TOOLONGNAME",        // 30 
  "F_ERR_NOTFORREAD",         // 31 
  "F_ERR_DELFUNC",            // 32 
  "F_ERR_ALLOCATION",         // 33 
  "F_ERR_INVALIDPOS",         // 34 
  "F_ERR_NOMORETASK",         // 35 
  "F_ERR_NOTAVAILABLE",       // 36 
  "F_ERR_TASKNOTFOUND",       // 37 
  "F_ERR_UNUSABLE"            // 38 
};


/*-------------------------------------------------------------------
  DisplayEffsErrorCode()
 -------------------------------------------------------------------*/
void DisplayEffsErrorCode( int code )
{
   if ( code <= MAX_EFFS_ERRORCODE )
   {
      iprintf( "%s\r\n", EffsErrorCode[code] );
   }
   else
   {
      iprintf( "Unknown EFFS error code [%d]\r\n", code );
   }
}


/*-------------------------------------------------------------------
  Init() - Initialization function for setting up EFFS on 
  MultiMedia/Secure Digital cards or Comact Flash cards.
 -------------------------------------------------------------------*/
BYTE InitExtFlash()
{

#if (defined USE_MMC)
   // Card detection check
   if ( get_cd() == 0 )
   {
      while ( get_cd() == 0 )
      {
         iprintf( "No MMC/SD card detected. Insert a card and then press " );
         iprintf( "<Enter>\r\n" );
         if ( getchar() == 13 )
         {
            break;
         }
      }
   }
   // Write protection check
   if ( get_wp() == 1 )
   {
      while ( get_wp() == 1 )
      {
         iprintf( "SD/MMC Card is write-protected. Disable write protection then " );
         iprintf( "press <Enter>\r\n" );
         if ( getchar() == 13 )
         {
            break;
         }
      }
   }
#endif
   
   int rv;
   
   /* The f_mountfat() function is called to mount a FAT drive. Although
      there are paramters in this function, they should not be modified. 
      The function call should be used as it appears for a Compact Flash
      card. For reference, the parameters are:
      
      drive_num:   Set to MMC_DRV_NUM, the drive to be mounted
      p_init_func: Points to the initialization function for the drive. 
                   For the Compact Flash drive, the function is located
                   in \nburn\<platform>\system\cfc_mem_nburn.c.
      p_user_info: Used to pass optional information. In this case, the
                   drive number.
      
      The return values are:
         F_NO_ERROR:  drive successfully mounted
         Any other value: Error occurred. See page 22 in the HCC-Embedded
         file system manual for the list of error codes.
   */ 
   siprintf(driveType, "No Drive");   
#if (defined USE_MMC)
   siprintf(driveType, "SD/MMC");   
   rv = f_mountfat( MMC_DRV_NUM, mmc_initfunc, F_MMC_DRIVE0 );
#elif (defined USE_CFC)
   siprintf(driveType, "CFC");   
   rv = f_mountfat( CFC_DRV_NUM, cfc_initfunc, F_CFC_DRIVE0 );
#endif
   if ( rv == F_NO_ERROR )
   {
      iprintf( "FAT mount to %s successful\r\n", driveType );
   }
   else
   {
      iprintf( "FAT mount to %s failed: ", driveType );
      DisplayEffsErrorCode( rv );
      return rv;
   }

   /* Change to SD/MMC drive
      We need to call the change function to access the new drive. Note
      that ANY function other than the f_mountfat() is specific to a task.
      This means that if f_chdrive() is called in a different task to a
      different drive, it will not affect this task - this task will 
      remain on the same drive.
   */

   rv = f_chdrive( EXT_FLASH_DRV_NUM );

   if ( rv == F_NO_ERROR )
   {
      iprintf( "%s drive change successful\r\n", driveType );
   }
   else
   {
      iprintf( "%s drive change failed: ", driveType );
      DisplayEffsErrorCode( rv );
   }
   
   return rv;  
}

/*-------------------------------------------------------------------
 DWORD ReadFile( BYTE* pReadBuffer, char* pFileName, DWORD NumBytes );
  - Open, Writes, Closes File and prints status, returns 0 for success
 
-------------------------------------------------------------------*/
DWORD ReadFile( BYTE* pReadBuffer, char* pFileName, DWORD NumBytes )
{
   DWORD rvR, rvC;
   F_FILE* fp = f_open( pFileName, "r" ); 
   if ( fp )
   {
      rvR = (DWORD)f_read( pReadBuffer, 1, NumBytes, fp );
      //if( NumBytes != rvR )
      //    iprintf( "*** Warning in ReadFile(%s): %d out of %d bytes read\r\n", pFileName, rvR, NumBytes );

      rvC = f_close( fp );  // Close a previously opened file of type F_FILE
      if ( rvC != F_NO_ERROR )
      {
         iprintf( "*** Error in f_close(%s): ", pFileName );
         DisplayEffsErrorCode( rvC );
         return 0;
      }
   }
   else
   {
	   iprintf("Failed to open file %s\r\n", pFileName);
      return 0;
   }
   return rvR;
}

